@@ -1,16 +1,16 @@
 use crate::abi::FnAbiLlvmExt;
-use crate::attributes::{self, Symbols};
+use crate::attributes::{self, NvvmAttributes, Symbols};
 use crate::debug_info::{self, compile_unit_metadata, CrateDebugContext};
 use crate::llvm::{self, BasicBlock, Type, Value};
 use crate::{target, LlvmMod};
 use nvvm::NvvmOption;
-use rustc_codegen_ssa::traits::ConstMethods;
 use rustc_codegen_ssa::traits::{BackendTypes, BaseTypeMethods, CoverageInfoMethods, MiscMethods};
+use rustc_codegen_ssa::traits::{ConstMethods, DerivedTypeMethods};
 use rustc_data_structures::base_n;
 use rustc_hash::FxHashMap;
 use rustc_middle::dep_graph::DepContext;
 use rustc_middle::ty::layout::{
-    FnAbiError, FnAbiOf, FnAbiRequest, HasParamEnv, LayoutError, TyAndLayout,
+    FnAbiError, FnAbiOf, FnAbiRequest, HasParamEnv, HasTyCtxt, LayoutError, TyAndLayout,
 };
 use rustc_middle::ty::layout::{FnAbiOfHelpers, LayoutOfHelpers};
 use rustc_middle::ty::{Ty, TypeFoldable};
@@ -50,6 +50,7 @@ pub(crate) struct CodegenCx<'ll, 'tcx> {
     pub const_cstr_cache: RefCell<FxHashMap<Symbol, &'ll Value>>,
     /// A map of functions which have parameters at specific indices replaced with an int-remapped type.
     /// such as i128 --> <2 x i64>
+    #[allow(clippy::type_complexity)]
     pub remapped_integer_args:
         RefCell<FxHashMap<&'ll Type, (Option<&'ll Type>, Vec<(usize, &'ll Type)>)>>,
 
@@ -91,13 +92,7 @@ pub(crate) struct CodegenCx<'ll, 'tcx> {
     eh_personality: &'ll Value,
 
     pub symbols: Symbols,
-
-    // we do not currently use codegen_args before linking, and during linking we reparse
-    // them because codegencx is not available at link time. However, we keep this so
-    // it is easier to use them in the future and add args we want to use before linking.
-    #[allow(dead_code)]
     pub codegen_args: CodegenArgs,
-
     // the value of the last call instruction. Needed for return type remapping.
     pub last_call_llfn: Cell<Option<&'ll Value>>,
 }
@@ -163,6 +158,7 @@ impl<'ll, 'tcx> CodegenCx<'ll, 'tcx> {
             symbols: Symbols {
                 nvvm_internal: Symbol::intern("nvvm_internal"),
                 kernel: Symbol::intern("kernel"),
+                addrspace: Symbol::intern("addrspace"),
             },
             dbg_cx,
             codegen_args: CodegenArgs::from_session(tcx.sess()),
@@ -183,7 +179,7 @@ impl<'ll, 'tcx> CodegenCx<'ll, 'tcx> {
 
     fn create_used_variable_impl(&self, name: *const i8, values: &[&'ll Value]) {
         let section = "llvm.metadata\0".as_ptr().cast();
-        let array = self.const_array(&self.type_ptr_to(self.type_i8()), values);
+        let array = self.const_array(self.type_ptr_to(self.type_i8()), values);
 
         unsafe {
             trace!(
@@ -223,7 +219,7 @@ impl<'ll, 'tcx> MiscMethods<'tcx> for CodegenCx<'ll, 'tcx> {
     }
 
     fn sess(&self) -> &Session {
-        &self.tcx.sess
+        self.tcx.sess
     }
 
     fn check_overflow(&self) -> bool {
@@ -266,6 +262,24 @@ impl<'ll, 'tcx> MiscMethods<'tcx> for CodegenCx<'ll, 'tcx> {
 }
 
 impl<'ll, 'tcx> CodegenCx<'ll, 'tcx> {
+    /// Computes the address space for a static.
+    pub fn static_addrspace(&self, instance: Instance<'tcx>) -> AddressSpace {
+        let ty = instance.ty(self.tcx, ty::ParamEnv::reveal_all());
+        let is_mutable = self.tcx().is_mutable_static(instance.def_id());
+        let attrs = self.tcx.get_attrs(instance.def_id());
+        let nvvm_attrs = NvvmAttributes::parse(self, attrs);
+
+        if let Some(addr) = nvvm_attrs.addrspace {
+            return AddressSpace(addr as u32);
+        }
+
+        if !is_mutable && self.type_is_freeze(ty) {
+            AddressSpace(4)
+        } else {
+            AddressSpace::DATA
+        }
+    }
+
     /// Declare a global value, returns the existing value if it was already declared.
     pub fn declare_global(
         &self,
@@ -289,7 +303,7 @@ impl<'ll, 'tcx> CodegenCx<'ll, 'tcx> {
 
     /// Declare a function. All functions use the default ABI, NVVM ignores any calling convention markers.
     /// All functions calls are generated according to the PTX calling convention.
-    /// https://docs.nvidia.com/cuda/nvvm-ir-spec/index.html#calling-conventions
+    /// <https://docs.nvidia.com/cuda/nvvm-ir-spec/index.html#calling-conventions>
     pub fn declare_fn(
         &self,
         name: &str,
@@ -324,10 +338,10 @@ impl<'ll, 'tcx> CodegenCx<'ll, 'tcx> {
         ty: &'ll Type,
         address_space: AddressSpace,
     ) -> Option<&'ll Value> {
-        if self.get_defined_value(&name).is_some() {
+        if self.get_defined_value(name).is_some() {
             None
         } else {
-            Some(self.declare_global(&name, ty, address_space))
+            Some(self.declare_global(name, ty, address_space))
         }
     }
 
@@ -416,7 +430,7 @@ impl<'ll, 'tcx> CodegenCx<'ll, 'tcx> {
 
         let abi = self.fn_abi_of_instance(instance, ty::List::empty());
 
-        let llfn = if let Some(llfn) = self.get_declared_value(&sym) {
+        let llfn = if let Some(llfn) = self.get_declared_value(sym) {
             trace!("Returning existing llfn `{:?}`", llfn);
             let llptrty = abi.ptr_to_llvm_type(self);
 
@@ -431,7 +445,7 @@ impl<'ll, 'tcx> CodegenCx<'ll, 'tcx> {
                 llfn
             }
         } else {
-            let llfn = self.declare_fn(&sym, abi.llvm_type(self), Some(abi));
+            let llfn = self.declare_fn(sym, abi.llvm_type(self), Some(abi));
             attributes::from_fn_attrs(self, llfn, instance);
             let def_id = instance.def_id();
 
@@ -504,25 +518,31 @@ impl<'ll, 'tcx> CodegenCx<'ll, 'tcx> {
     }
 }
 
+#[derive(Default, Clone)]
 pub struct CodegenArgs {
     pub nvvm_options: Vec<NvvmOption>,
+    pub override_libm: bool,
 }
 
 impl CodegenArgs {
     pub fn from_session(sess: &Session) -> Self {
-        match Self::parse(&sess.opts.cg.llvm_args) {
-            Ok(x) => x,
-            Err(err) => sess.fatal(&format!("Failed to parse codegen args: {}", err)),
-        }
+        Self::parse(&sess.opts.cg.llvm_args)
     }
 
     // we may want to use rustc's own option parsing facilities to have better errors in the future.
-    pub fn parse(args: &[String]) -> Result<Self, &'static str> {
-        let nvvm_options = args
-            .iter()
-            .map(|x| NvvmOption::from_str(x))
-            .collect::<Result<Vec<_>, _>>()?;
-        Ok(Self { nvvm_options })
+    pub fn parse(args: &[String]) -> Self {
+        // TODO: replace this with a "proper" arg parser.
+        let mut cg_args = Self::default();
+
+        for arg in args {
+            if let Ok(flag) = NvvmOption::from_str(arg) {
+                cg_args.nvvm_options.push(flag);
+            } else if arg == "--override-libm" {
+                cg_args.override_libm = true;
+            }
+        }
+
+        cg_args
     }
 }
 
